#!/usr/bin/env python
# -*- coding: utf-8 -*-

# file      : code_gen.py
# project   : basebuildpy
# copyright : (C) 2014 by Claude Bulin
#
# Generating a skeleton python from one or more files *. Glade or *. Ui
#
# This file is part of basebuildpy project
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import os, stat, subprocess, shutil, tarfile
from datetime import date
from fconfig import FConfig


class codegen_py():

	def __init__(self):
		self.ListeDouble = []


	#
	# write
	#
	def write_block_control( self, file_object, text_control, text_print, initial_indent=0 ):
		if not text_control in self.ListeDouble:
			self.ListeDouble.append( text_control )
			file_object.write( '\t'*1*initial_indent+text_print + '\n' )
			return True
		return False

	def write_block( self, file_object, text_print, initial_indent=0 ):
		file_object.write( '\t'*1*initial_indent+text_print + '\n' )


	#
	# collect_tags
	#
	def collect_tags(self, xmlfile):
		'''collect signal handlers and control objects from the xml file'''
		import xml.dom.minidom as dom
		doc = dom.parse(xmlfile)

		tag_collection={}

		# search new format: <object class="GtkWindow" id="wind_main">
		tag_collection['CLAUDE'] = \
			[object.getAttribute('id') for object in doc.getElementsByTagName('object')\
			 if object.getAttribute('class') in \
			 ['GtkFileChooserDialog','GtkWindow','GtkDialog','GtkAboutDialog']]
		# search old format: <widget class="GtkWindow" id="window1">
		tag_collection['CLAUDE'] += \
			[object.getAttribute('id') for object in doc.getElementsByTagName('widget')\
			 if object.getAttribute('class') in \
			 ['GtkFileChooserDialog','GtkWindow','GtkDialog','GtkAboutDialog']]

		# search format: <signal name="activate" handler="on_menuitem_quit_activate" swapped="no"/>
		tag_collection['signalhandlers'] = \
			[signal.getAttribute('handler') + ' ' + signal.getAttribute('name') for signal in doc.getElementsByTagName('signal')]

		# search new format: <object class="GtkSpinButton" id="spinbutton_exemple">
		tag_collection['controlobjects'] = \
			[object.getAttribute('id') for object in doc.getElementsByTagName('object')\
			 if object.getAttribute('class') in \
			 ['GtkEntry','GtkTextView','GtkComboBox','GtkComboBoxEntry','GtkProgressBar',\
			  'GtkCheckbox','GtkRadioButton','GtkSpinButton','GtkCalendar','GtkToolBar', 'GtkCheckButton', 'GtkButton', \
			  'GtkStatusbar','GtkFileChooserButton','GtkFontButton','GtkHScale','GtkVScale','GtkLabel','GtkAboutDialog','GtkFileChooserDialog']]
		# search old format: <widget class="GtkSpinButton" id="sb_heure">
		tag_collection['controlobjects'] += \
			[object.getAttribute('id') for object in doc.getElementsByTagName('widget')\
			 if object.getAttribute('class') in \
			 ['GtkEntry','GtkTextView','GtkComboBox','GtkComboBoxEntry','GtkProgressBar',\
			  'GtkCheckbox','GtkRadioButton','GtkSpinButton','GtkCalendar','GtkToolBar', 'GtkCheckButton', 'GtkButton', \
			  'GtkStatusbar','GtkFileChooserButton','GtkFontButton','GtkHScale','GtkVScale','GtkLabel','GtkAboutDialog','GtkFileChooserDialog']]

		return tag_collection


	def write_entete(self, fp, p_Name, p_ProjectName, p_ProjectAuthor ):
		# init code
		self.write_block( fp, '#!/usr/bin/env python\n# -*- coding: utf-8 -*-\n# \n', 0 )
		# license
		self.write_block( fp, '# file      : %s.py' % p_Name, 0 )
		self.write_block( fp, '# project   : %s' % p_ProjectName, 0 )
		self.write_block( fp, '# copyright : (C) %s by %s' % (date.today().year, p_ProjectAuthor), 0 )
		self.write_block( fp, '# ', 0 )
		self.write_block( fp, '# This file is part of %s project' % p_ProjectName, 0 )
		self.write_block( fp, '# ', 0 )
		self.write_block( fp, '# %s is free software; you can redistribute it and/or' % p_ProjectName, 0 )
		self.write_block( fp, '# modify it under the terms of the GNU Lesser General Public', 0 )
		self.write_block( fp, '# License as published by the Free Software Foundation; version 3.', 0 )
		self.write_block( fp, '#', 0 )
		self.write_block( fp, '# %s is distributed in the hope that it will be useful,' % p_ProjectName, 0 )
		self.write_block( fp, '# but WITHOUT ANY WARRANTY; without even the implied warranty of', 0 )
		self.write_block( fp, '# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU', 0 )
		self.write_block( fp, '# Lesser General Public License for more details.', 0 )
		self.write_block( fp, '#', 0 )
		self.write_block( fp, '# You should have received a copy of the GNU Lesser General Public', 0 )
		self.write_block( fp, '# License along with this library; if not, write to the Free Software', 0 )
		self.write_block( fp, '# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307', 0 )
		self.write_block( fp, '# USA\n\n', 0 )


	#
	#
	#
	def create_template( self, p_ProjectName, p_ProjectVersion, p_ProjectAuthor, p_ProjectDestination, p_ListNameFilesGlade ):

		PathDest      = p_ProjectDestination + "/" + p_ProjectName + "-" + p_ProjectVersion
		PathDestSrc   = PathDest + "/src"
		PathDestGlade = PathDest + "/data"
		PathDestMan   = PathDest + "/man"

		# create directories
		os.makedirs( PathDest )
		os.makedirs( PathDestSrc )
		os.makedirs( PathDestGlade )
		os.makedirs( PathDestMan )
		# copy glade file to dest
		for GladeFile in p_ListNameFilesGlade:
			BaseName = os.path.basename( GladeFile )
			Name = BaseName.split(".")[0]
			Ext  = BaseName.split(".")[-1]
			PathDestNameGlade = PathDestGlade + "/" + Name + ".glade"

			shutil.copy2( GladeFile, PathDestNameGlade )

			#
			# TODO: This precaution is useful ???
			#
			# REPLACE glade-interface WITH interface
			command = "sed -i 's/glade-interface/interface/g' " + PathDestNameGlade
			# print( " "*4, command )
			subprocess.call(command, shell=True)
			# REPLACE widget WITH object
			command = "sed -i 's/widget/object/g' " + PathDestNameGlade
			# print( " "*4, command )
			subprocess.call(command, shell=True)
			# REPLACE action-widget WITH action-object
			command = "sed -i 's/action-object/action-widget/g' " + PathDestNameGlade
			# print( " "*4, command )
			subprocess.call(command, shell=True)


		#
		# create py file in src/
		#
		for GladeFile in p_ListNameFilesGlade:

			# Fetch collected tags
			tags = self.collect_tags( GladeFile )

			BaseName = os.path.basename( GladeFile )
			Name = BaseName.split(".")[0]
			Ext  = BaseName.split(".")[-1]
			fileobj = open( PathDestSrc + "/" + Name + ".py", 'w' )

			if len(tags['CLAUDE']) == 0:
				print( ">>> Sorry but no windows attribute in this file !" )
			else:
				self.write_entete( fileobj, Name, p_ProjectName, p_ProjectAuthor )
				# import
				self.write_block( fileobj, 'import sys, os', 0 )
				self.write_block( fileobj, 'try:', 0 )
				self.write_block( fileobj, 'from gi.repository import Gtk, GObject', 1 )
				self.write_block( fileobj, 'except:', 0 )
				self.write_block( fileobj, 'print("GTK Not Availible")', 1 )
				self.write_block( fileobj, 'sys.exit(1)', 1 )
				for handler in tags['CLAUDE'] :
					root_wind = handler
					break
				# import
				self.write_block( fileobj, 'from fconfig import FConfig', 0 )
				self.write_block( fileobj, 'import Language_Init', 0 )
				self.write_block( fileobj, '', 0 )
				self.write_block( fileobj, '', 0 )
				# class
				self.write_block( fileobj, 'class %s():'% Name, 0 )
				# __init__
				self.write_block( fileobj, '', 0 )
				self.write_block( fileobj, 'def __init__( self, name_glade_file="%s.glade" ):' % Name, 1 )
				self.write_block( fileobj, '# def __init__( self, name_glade_file="%s.glade", Form=None ):' % Name, 1 )
				self.write_block( fileobj, '', 0 )
				# language
				self.write_block( fileobj, '# Add language support' , 2 )
				self.write_block( fileobj, '_ = Language_Init.Translator(FConfig).lang.gettext' , 2 )
				self.write_block( fileobj, 'self._ = _' , 2 )
				self.write_block( fileobj, '', 2 )
				# try Gtk.Builder()
				self.write_block( fileobj, 'try:', 2 )
				self.write_block( fileobj, 'self.builder = Gtk.Builder()', 3 )
				self.write_block( fileobj, '# Add language support from glade file', 3 )
				self.write_block( fileobj, 'self.builder.set_translation_domain("%s")' % p_ProjectName, 3 )
				self.write_block( fileobj, 'self.builder.add_from_file( FConfig.DATADIR + "/" + name_glade_file )', 3 )
				self.write_block( fileobj, 'except:', 2 )
				self.write_block( fileobj, 'print("except from:Gtk.Builder() ")', 3 )
				self.write_block( fileobj, 'sys.exit(1)\n', 3 )
				# get handler win
				for handler in tags['CLAUDE'] :
					self.write_block( fileobj, 'self.%s=self.builder.get_object("%s")' % (handler, handler), 2 )
					self.write_block( fileobj, '# if Form != None:', 2 )
					self.write_block( fileobj, '#	self.%s.set_transient_for( Form )' % handler, 2 )
					self.write_block( fileobj, '#	self.%s.set_modal( True )' % handler, 2 )
				self.write_block( fileobj, '', 2 )
				# dictionary of events
				self.ListeDouble[:] = []
				self.write_block( fileobj, 'dic={', 2 )
				for signal in tags['signalhandlers']:
					SpltSignal = []
					SpltSignal = signal.split(' ')
					self.write_block_control( fileobj, signal ,'"%s": self.%s,' %(SpltSignal[0],SpltSignal[0]), initial_indent=3 )
					SpltSignal[:] = []
				self.write_block( fileobj, '}', 2 )
				self.write_block( fileobj, 'self.builder.connect_signals(dic)', 2 )
				# get handler obj in win
				self.write_block( fileobj, '', 2 )
				for handler in tags['controlobjects'] :
					self.write_block( fileobj, 'self.%s=self.builder.get_object("%s")' % (handler, handler), 2 )
				self.write_block( fileobj, '', 2 )
				#
				for handler in tags['controlobjects'] :
					if handler == 'chooserdialog':
						self.write_block( fileobj, 'self.chooserdialog.set_default_response (Gtk.ResponseType.OK)', 2 )
						self.write_block( fileobj, 'self.chooserdialog.set_local_only(False)', 2 )
						self.write_block( fileobj, 'self.chooserdialog.set_modal(True)', 2 )
						self.write_block( fileobj, 'self.chooserdialog.set_action(Gtk.FileChooserAction.SELECT_FOLDER)', 2 )
						self.write_block( fileobj, 'self.chooserdialog.set_select_multiple(False)', 2 )
						self.write_block( fileobj, '# self.chooserdialog.set_current_folder( <PATH> )', 2 )
						break
				# header
				self.write_block( fileobj, '# header', 2 )
				self.write_block( fileobj, '# title_string = _("SET HEADER HERE")', 2 )
				self.write_block( fileobj, '# string = "<span font_desc=\\"sans bold 12 \\"><b>\\n\\n%s\\n</b></span>" % title_string', 2 )
				self.write_block( fileobj, '# self.label_title_wind_main.set_markup( string )', 2 )
				# title wind
				self.write_block( fileobj, 'self.%s' % root_wind + '.set_title("%s - Version %s -  by %s" % (FConfig.__PACKAGE__, FConfig.__VERSION__, FConfig.__AUTHOR__))', 2 )
				self.write_block( fileobj, '# self.%s' % root_wind + '.set_icon_from_file( FConfig.DATADIR + "/<NAME_IMAGE>.png" )', 2 )
				# show
				self.write_block( fileobj, 'self.%s.show()' % root_wind, 2 )
				self.write_block( fileobj, '', 2 )
				# def functions for dictionary of events
				self.ListeDouble[:] = []
				for signal in tags['signalhandlers']:
					SpltSignal = []
					SpltSignal = signal.split(' ')
					#
					# def on_notebook_choice_switch_page(self, obj1, obj2, page):
					#
					if "realize" in SpltSignal[1]:
						definition = 'def %s(self, widget, data=None):' % SpltSignal[0]
					elif "draw" in SpltSignal[1]:
						definition = 'def %s(self, widget, cr):' % SpltSignal[0]
					elif "switch-page" in SpltSignal[1] or "switch_page" in SpltSignal[1]:
						definition = 'def %s(self, obj1, obj2, page):' % SpltSignal[0]
					else:
						definition = 'def %s(self, widget, data=None):' % SpltSignal[0]
					if self.write_block_control( fileobj, SpltSignal[0] , definition, initial_indent=1 ):
						self.write_block( fileobj, 'print( "%s" )' % SpltSignal[0], 2 )
						if "delete-event" in SpltSignal[1] or "destroy-event" in SpltSignal[1]:
							self.write_block( fileobj, 'Gtk.main_quit()', 2 )
							self.write_block( fileobj, '# self.%s.destroy()' % Name, 2 )
						self.write_block( fileobj, '', 2 )
					SpltSignal[:] = []
				# run
				self.write_block( fileobj, "\nif __name__ == '__main__':", 0)
				self.write_block( fileobj, 'wind = %s()'% Name, 1 )
				self.write_block( fileobj, 'Gtk.main()', 1 )

			fileobj.close()
			# for the next file
			del tags

		#
		# create p_ProjectName.py for test code
		#
		NameMainFile = PathDest + '/%s.py' % p_ProjectName
		MainFile = open( NameMainFile, "w" )
		self.write_entete( MainFile, p_ProjectName, p_ProjectName, p_ProjectAuthor )
		# import
		MainFile.write( 'import sys, os\n' )
		MainFile.write( 'try:\n' )
		MainFile.write( '\tfrom gi.repository import Gtk, GObject\n' )
		MainFile.write( 'except:\n' )
		MainFile.write( '\tprint("GTK Not Availible")\n' )
		MainFile.write( '\tsys.exit(1)\n' )
		MainFile.write( "\n" )
		MainFile.write( "from fconfig import FConfig\n" )
		num_file = 1
		for GladeFile in p_ListNameFilesGlade:
			BaseName = os.path.basename( GladeFile )
			Name = BaseName.split(".")[0]
			if num_file == 1:
				MainFile.write( "from " + Name + " import " + Name + "\n" )
			else:
				MainFile.write( "# from " + Name + " import " + Name + "\n" )
			num_file += 1
		MainFile.write( "\n" )
		# run
		MainFile.write( "if __name__ == '__main__':\n" )
		MainFile.write( '\tprint( "")\n' )
		MainFile.write( '\tprint( "!"+"-"*40)\n' )
		MainFile.write( '\tprint( "! %s - %s" % (FConfig.__PACKAGE__,  FConfig.__VERSION__))\n' )
		MainFile.write( '\tprint( "!"+"-"*3)\n' )
		MainFile.write( '\tprint( "! <Program description>" )\n' )
		MainFile.write( '\tprint( "!"+"-"*3)\n' )
		MainFile.write( '\tprint( "! %s" % FConfig.__AUTHOR__)\n' )
		MainFile.write( '\tprint( "! %s" % FConfig.__URL__ )\n' )
		MainFile.write( '\tprint( "! Licence GPL-3" )\n' )
		MainFile.write( '\tprint( "!"+"-"*40)\n' )
		MainFile.write( "\tFConfig.print_FConfig()\n" )
		num_file = 1
		for GladeFile in p_ListNameFilesGlade:
			BaseName = os.path.basename( GladeFile )
			Name = BaseName.split(".")[0]
			if num_file == 1:
				MainFile.write( "\ttst = " + Name + "()\n" )
			else:
				MainFile.write( "\t# tst = " + Name + "()\n" )
			num_file += 1
		MainFile.write( "\tGtk.main()\n" )
		MainFile.write( "\n" )
		MainFile.close()

		# chmod +x NameMainFile
		st = os.stat( NameMainFile )
		os.chmod( NameMainFile, st.st_mode | stat.S_IEXEC)

		#
		# create __init__.py for test code
		#
		NameMainFile = PathDest + '/__init__.py'
		MainFile = open( NameMainFile, "w" )
		MainFile.close()

		#
		# create fconfig.py for test code
		#
		NameMainFile = PathDest + '/fconfig.py'
		MainFile = open( NameMainFile, "w" )
		self.write_entete( MainFile, 'fconfig', p_ProjectName, p_ProjectAuthor )
		# import
		MainFile.write( 'import sys, os\n' )
		# class
		MainFile.write( '\n' )
		MainFile.write( 'class FConfig:\n' )
		MainFile.write( '\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# name of package\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	__PACKAGE__ = "%s"\n' % p_ProjectName)
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# version\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	__VERSION__ = "%s"\n' % p_ProjectVersion )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# authors\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	__AUTHOR__  = "%s"\n' % p_ProjectAuthor )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# adr mail\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	__MAIL__    = "<%s dot gmail point com>"\n' % p_ProjectAuthor )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# url\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	__URL__     = "<URL>"\n' )
		MainFile.write( '\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# def du fichier de config de %s\n' % p_ProjectName )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	CONFIGFILE = os.getenv("XDG_CONFIG_HOME",\n' )
		MainFile.write( '					   os.path.join(os.getenv("HOME"), ".config", "%s" )\n' % p_ProjectName )
		MainFile.write( '					   )\n' )
		MainFile.write( '	if not os.path.isdir(CONFIGFILE):\n' )
		MainFile.write( '		os.makedirs(CONFIGFILE)\n' )
		MainFile.write( '\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# def du repertoire contenant les data de %s\n' % p_ProjectName )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	DATADIR = os.path.join( os.path.dirname(__file__),"data")\n' )
		MainFile.write( '	if not os.path.isdir(DATADIR):\n' )
		MainFile.write( '		DATADIR = "/usr/share/%s"\n' % p_ProjectName )
		MainFile.write( '\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# def du repertoire contenant les sources de %s\n' % p_ProjectName )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	SRCDIR = os.path.join( os.path.dirname(__file__),"src")\n' )
		MainFile.write( '	if os.path.isdir(SRCDIR):\n' )
		MainFile.write( '		sys.path.append(SRCDIR)\n' )
		MainFile.write( '	else:\n' )
		MainFile.write( '		SRCDIR = "/usr/share/%s"\n' % p_ProjectName )
		MainFile.write( '		sys.path.append(SRCDIR)\n' )
		MainFile.write( '\n' )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	# def du repertoire contenant les traductions de %s\n' % p_ProjectName )
		MainFile.write( '	#================================================================\n' )
		MainFile.write( '	LOCALE_DIR = "/usr/share/locale/"\n' )
		MainFile.write( '	sys.path.append(LOCALE_DIR)\n' )
		MainFile.write( '\n' )
		MainFile.write( '	def print_FConfig():\n' )
		MainFile.write( '		print('')\n' )
		MainFile.write( '		print("CONFIGFILE ", FConfig.CONFIGFILE)\n' )
		MainFile.write( '		print("DATADIR    ", FConfig.DATADIR)\n' )
		MainFile.write( '		print("SRCDIR     ", FConfig.SRCDIR)\n' )
		MainFile.write( '		print("LOCALE_DIR ", FConfig.LOCALE_DIR)\n' )
		MainFile.close()

		#
		# create src/__init__.py
		#
		NameMainFile = PathDestSrc + '/__init__.py'
		MainFile = open( NameMainFile, "w" )
		MainFile.close()

		#
		# create src/prefs.py
		#
		NameMainFile = PathDestSrc + '/prefs.py'
		MainFile = open( NameMainFile, "w" )
		self.write_entete( MainFile, 'prefs', p_ProjectName, p_ProjectAuthor )
		# import
		MainFile.write( '# TODO -> A TERMINER !!!\n' )
		MainFile.close()

		#
		# create src/Language_Init.py
		#
		NameMainFile = PathDestSrc + '/Language_Init.py'
		MainFile = open( NameMainFile, "w" )
		self.write_entete( MainFile, 'Language_Init', p_ProjectName, p_ProjectAuthor )
		# import
		MainFile.write( 'from gi.repository import Gtk\n' )
		MainFile.write( 'import os, sys\n' )
		MainFile.write( 'import gettext, locale\n' )
		# class
		MainFile.write( '\n' )
		MainFile.write( 'class Translator:\n' )
		MainFile.write( '	def __init__(self, project):\n' )
		MainFile.write( '		self.project = project\n' )
		MainFile.write( '		# Setup foreign language support\n' )
		MainFile.write( '		langs = []\n' )
		MainFile.write( '		lc, encoding = locale.getlocale(locale.LC_MESSAGES)\n' )
		MainFile.write( '		if (lc):\n' )
		MainFile.write( '			langs = [lc]\n' )
		MainFile.write( '		# Add by xcfaudio: complementary xx from xx_XX\n' )
		MainFile.write( '		if len(lc) == 5:\n' )
		MainFile.write( '			str_lang = lc[0:2]\n' )
		MainFile.write( '			if str_lang not in langs:\n' )
		MainFile.write( '				langs.append( str_lang )\n' )
		MainFile.write( '		language = os.environ.get("LANGUAGE", None)\n' )
		MainFile.write( '		if (language):\n' )
		MainFile.write( '			langs += language.split(":")\n' )
		MainFile.write( '		# Set locale specific settings\n' )
		MainFile.write( '		locale.setlocale(locale.LC_ALL)\n' )
		MainFile.write( '		locale.bindtextdomain("%s", self.project.LOCALE_DIR)\n' % p_ProjectName )
		MainFile.write( '		locale.bind_textdomain_codeset("%s", "UTF-8")\n' % p_ProjectName )
		MainFile.write( '		# Set gettext settings\n' )
		MainFile.write( '		gettext.bindtextdomain("%s", self.project.LOCALE_DIR)\n' % p_ProjectName )
		MainFile.write( '		gettext.bind_textdomain_codeset("%s", "UTF-8")\n' % p_ProjectName )
		MainFile.write( '		gettext.textdomain("%s")\n' % p_ProjectName )
		MainFile.write( '		gettext.install(domain="%s", localedir=self.project.LOCALE_DIR)\n' % p_ProjectName )
		MainFile.write( '		# This reference is used by other classes to define the _ method\n' )
		MainFile.write( '		self.lang = gettext.translation("%s", self.project.LOCALE_DIR, languages = langs, fallback = True)\n' % p_ProjectName )
		MainFile.close()

		#
		# create AUTHORS
		#
		if os.path.isfile( "/usr/bin/figlet" ) == True:
			command = 'figlet -w 200 -f small "    %s"' % p_ProjectName + ' > %s/' % PathDest + 'AUTHORS'
			subprocess.call(command, shell=True)
		else:
			NameMainFile = PathDest + '/AUTHORS'
			MainFile = open( NameMainFile, "w" )
			MainFile.write( '!---\n' )
			MainFile.write( '! %s : AUTHORS\n' % p_ProjectName )
			MainFile.write( '!---\n' )
			MainFile.close()

		#
		# create ChangeLog
		#
		if os.path.isfile( "/usr/bin/figlet" ) == True:
			command = 'figlet -w 200 -f small "    %s"' % p_ProjectName + ' > %s/' % PathDest + 'ChangeLog'
			subprocess.call(command, shell=True)
		else:
			NameMainFile = PathDest + '/ChangeLog'
			MainFile = open( NameMainFile, "w" )
			MainFile.write( '!---\n' )
			MainFile.write( '! %s : ChangeLog\n' % p_ProjectName )
			MainFile.write( '!---\n' )
			MainFile.close()

		NameMainFile = PathDest + '/ChangeLog'
		MainFile = open( NameMainFile, "a" )
		MainFile.write( '\n' )
		MainFile.write( '\t========================================\n' )
		MainFile.write( '\t |                                    |\n' )
		MainFile.write( '\t |             Changelog              |\n' )
		MainFile.write( '\t |                                    |\n' )
		MainFile.write( '\t========================================\n' )
		MainFile.write( '\n' )
		MainFile.write( '\t========================================\n' )
		MainFile.write( '\t |  %s                             |\n' % p_ProjectVersion )
		MainFile.write( '\t========================================\n' )
		MainFile.write( '\t- Add comments\n' )
		MainFile.write( '\n' )
		MainFile.close()

		#
		# create COPYING
		#
		if os.path.isfile( "/usr/share/automake-1.14/COPYING" ) == True:
			shutil.copy2( "/usr/share/automake-1.14/COPYING", PathDest + "/COPYING" )

		#
		# create man/
		#
		FileMan = PathDestMan + "/%s.1" % p_ProjectName
		MainFileMan = open( FileMan, "w" )
		MainFileMan.write( '.\\" %s\n' % p_ProjectName )
		MainFileMan.write( '.\\" Copyright (C) %s %s\n' % (date.today().year, p_ProjectAuthor) )
		MainFileMan.write( '.\\"\n' )
		MainFileMan.write( '.\\" %s! is free software; you can redistribute it and/or modify\n' % p_ProjectName )
		MainFileMan.write( '.\\" it under the terms of the GNU General Public License as published by\n' )
		MainFileMan.write( '.\\" the Free Software Foundation; either version 2, or\n' )
		MainFileMan.write( '.\\" (at your option) any later version.\n' )
		MainFileMan.write( '.\\"\n' )
		MainFileMan.write( '.\\" %s! is distributed in the hope that it will be useful,\n'  % p_ProjectName )
		MainFileMan.write( '.\\" but WITHOUT ANY WARRANTY; without even the implied warranty of\n' )
		MainFileMan.write( '.\\" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU\n' )
		MainFileMan.write( '.\\" General Public License for more details.\n' )
		MainFileMan.write( '.\\"\n' )
		MainFileMan.write( '.\\" You should have received a copy of the GNU General Public License\n' )
		MainFileMan.write( '.\\" along with program; see the file COPYING. If not, write to the\n' )
		MainFileMan.write( '.\\" Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,\n' )
		MainFileMan.write( '.\\" MA 02111-1307, USA.\n' )
		MainFileMan.write( '.\\"\n' )
		MainFileMan.write( '\n' )
		MainFileMan.write( '.TH %s 1 "Thu, 06 Mar 2014" %s "Manual: %s"\n' % (p_ProjectName, p_ProjectVersion, p_ProjectVersion))
		MainFileMan.write( '\n' )
		MainFileMan.write( '.SH NAME\n' )
		MainFileMan.write( '\\fB%s\\fP \- Short description of the program\n' % p_ProjectName )
		MainFileMan.write( '.br\n' )
		MainFileMan.write( '\n' )
		MainFileMan.write( '.SH AUTHOR\n' )
		MainFileMan.write( '\\fB%s\\fP is developed by %s\n' % (p_ProjectName, p_ProjectAuthor) )
		MainFileMan.close()
		#
		# help text man
		#
		FileMan = PathDestMan + "/manpage.txt"
		MainFileMan = open( FileMan, "w" )
		MainFileMan.write( '\n' )
		MainFileMan.write( 'nroff -man ./%s.1 | less\n' % p_ProjectName )
		MainFileMan.write( 'gzip --best ./%s.1\n' % p_ProjectName )
		MainFileMan.write( '\n' )
		MainFileMan.close()

		#
		# create data/file.desktop
		#
		FileDesktop = PathDestGlade + "/%s.desktop" % p_ProjectName
		MainFileDesktop = open( FileDesktop, "w" )
		Description = _("Short description of the program")
		MainFileDesktop.write( '#!/usr/bin/env xdg-open\n' )
		MainFileDesktop.write( '[Desktop Entry]\n' )
		MainFileDesktop.write( 'Name=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'Name[fr]=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'Name[en]=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'Name[en_GB]=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'Comment=%s\n' % Description )
		MainFileDesktop.write( 'Comment[fr]=%s\n' % Description )
		MainFileDesktop.write( 'Comment[en]=%s\n' % Description )
		MainFileDesktop.write( 'Comment[en_GB]=%s\n' % Description )
		MainFileDesktop.write( 'GenericName=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'GenericName[fr]=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'GenericName[en]=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'GenericName[en_GB]=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'Icon=%s.png\n' % p_ProjectName)
		MainFileDesktop.write( 'Type=Application\n' )
		MainFileDesktop.write( 'Categories=Application;Development\n' )
		MainFileDesktop.write( 'Encoding=UTF-8\n' )
		MainFileDesktop.write( 'Exec=%s\n' % p_ProjectName )
		MainFileDesktop.write( 'Terminal=false\n' )
		MainFileDesktop.write( 'MultipleArgs=false\n' )
		MainFileDesktop.write( 'StartupNotify=false\n' )
		MainFileDesktop.write( 'X-KDE-SubstituteUID=false\n' )
		MainFileDesktop.write( 'X-KDE-Username=\n' )
		MainFileDesktop.close()















